# -*- coding: utf-8 -*-

from django.db import models
from django.conf import settings
from django.urls import reverse
from django.contrib.auth.models import BaseUserManager, AbstractBaseUser, PermissionsMixin
from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django.core.exceptions import SuspiciousOperation
from django.utils.crypto import get_random_string

from ldapaccounts.models import LdapUser, LdapGroup
from django.utils import timezone
from datetime import timedelta


__all__ = ["User", "Token", "Group", "AssociateToken"]


class UserManager(BaseUserManager):
    def create_user(self, email, password=None, first_name=None, last_name=None, login_inp_net=None, **extra_fields):
        email = UserManager.normalize_email(email)
        user = self.model(email=email, first_name=first_name, last_name=last_name, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        # 7 jours de délais avant la suppression par cron
        user.delete_date = timezone.now() + timedelta(days=7)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, first_name, last_name, password, **extra_fields):
        user = self.create_user(email, first_name, last_name, password, **extra_fields)
        user.is_superuser = True
        user.save(using=self._db)
        return user


class Group(models.Model):

    cn = models.CharField(max_length=128, unique=True)
    displayName = models.CharField(max_length=128)

    class Meta:
        ordering = ["displayName"]

    def __str__(self):
        return "{} - {}".format(self.displayName, self.cn)

    def get_LdapGroup(self):
        return LdapGroup.objects.get(cn=self.cn)


class User(AbstractBaseUser, PermissionsMixin):

    first_name = models.CharField(max_length=128, verbose_name="Prénom")
    last_name = models.CharField(max_length=128, verbose_name="Nom")
    promo = models.IntegerField(default=2000)
    email = models.EmailField(unique=True, error_messages={"unique": "Un compte existe déjà pour cette adresse email."})
    login_inp_net = models.CharField(max_length=128, blank=True)
    ecole = models.CharField(max_length=32, blank=True)
    is_adherent_ae = models.BooleanField(default=False)
    is_active = models.BooleanField(default=False)
    is_anonyme = models.BooleanField(default=False)
    managed_clubs = models.ManyToManyField(Group, related_name="managers")
    groupes = models.ManyToManyField(Group, related_name="users")
    # defini la date où il faut supprimer le compte exte
    delete_date = models.DateTimeField(auto_now_add=True)
    # mail_delete : défini l'état de suppression pour les comptes exté
    #   - si True : Un mail sera envoyé à la date de suppression (delete_date)
    #               La date de suppression sera programmé deux semaines après
    #   - si False : delete_date correspond à la date de suppression
    mail_delete = models.BooleanField(default=False)
    # Charte
    charte_date = models.DateTimeField(null=True)
    charte_version = models.IntegerField(null=True)

    objects = UserManager()

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = ["first_name", "last_name"]

    def get_full_name(self):
        if self.is_anonyme:
            return "Anonyme"
        return self.first_name + " " + self.last_name

    def get_short_name(self):
        if self.is_anonyme:
            return "Anonyme"
        return self.email

    def get_LdapUser(self):
        if self.login_inp_net:
            try:
                return LdapUser.objects.get(login_inp_net=self.login_inp_net)
            except LdapUser.DoesNotExist:
                return None
        return None

    def get_email(self):
        # A utiliser uniquement pour l'affichage à l'utilisateur
        if self.login_inp_net and self.get_LdapUser().mail_redirect:
            return self.get_LdapUser().mail_redirect
        return self.email

    def __str__(self):
        if self.is_anonyme:
            return "Anonyme"
        return self.get_short_name()

    @property
    def is_staff(self):
        return self.is_superuser

    @property
    def is_admin(self):
        return self.is_superuser

    @property
    def is_inp_net(self):
        return self.login_inp_net != ""

    @property
    def last_action(self):
        return self.logs.order_by("-date").first()

    def delete(self, *args, **kwargs):
        if not self.is_anonyme:
            super(User, self).delete(*args, **kwargs)
        else:
            raise SuspiciousOperation("Anonyme User can't be delete")

    def save(self, *args, **kwargs):
        if not self.is_anonyme:
            super(User, self).save(*args, **kwargs)
        else:
            raise SuspiciousOperation("Anonyme User can't be editate")

    @classmethod
    def get_anonyme_user(cls):
        anonyme_users = cls.objects.filter(is_anonyme=True)
        if anonyme_users.exists():
            return anonyme_users.first()

        anonyme_user = cls(
            first_name="Ano",
            last_name="Nyme",
            email="no_user@billetterie.inpt.fr",
            login_inp_net="",
            is_adherent_ae=False,
            is_active=False,
            is_anonyme=True,
        )
        super(cls, anonyme_user).save()
        return anonyme_user

    def can_delete(self):
        if not self.registrations.all():
            # pas de réservation
            return True
        date_last_event = self.registrations.order_by("-ticket__event__end_date").first().ticket.event.end_date
        if timezone.now() < date_last_event + timedelta(days=30):
            # si un billet est donné à l'utilisateur en attente de suppression,
            #     on lui renvoie un mail avant de supprimer son compte
            self.mail_delete = True
            self.save()
            return False
        return True


class Token(models.Model):

    USAGE_DEFINE_PASSWORD = 1
    USAGE_EMAIL_VALIDATION = 2
    USAGE_TYPE = (
        (USAGE_DEFINE_PASSWORD, "Password recovery"),
        (USAGE_EMAIL_VALIDATION, "Email validation"),
    )

    usage = models.IntegerField(choices=USAGE_TYPE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    uuid = models.CharField(max_length=64, unique=True)
    created_at = models.DateTimeField(auto_now_add=True)
    expire_at = models.DateTimeField(null=True, blank=True)
    used = models.BooleanField(default=False)
    payload = models.CharField(max_length=128, blank=True)

    def get_new_uuid(self):
        uuid = None
        while (not uuid) or Token.objects.filter(uuid=uuid).exists():
            uuid = get_random_string(length=32)
        self.uuid = uuid

    def __str__(self):
        return "{} token for {}".format(self.get_usage_display(), self.user)


class AssociateToken(models.Model):

    user = models.ForeignKey(User, on_delete=models.CASCADE)
    user_ldap = models.CharField(max_length=128)
    token_user = models.CharField(max_length=128, unique=True)
    token_user_ldap = models.CharField(max_length=128, unique=True)
    validation_user = models.BooleanField(default=False)
    validation_user_ldap = models.BooleanField(default=False)
    invalide = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)

    def get_user_ldap(self):
        return LdapUser.objects.get(login_inp_net=self.user_ldap)

    def send_mail(self):
        adresse_base = settings.BASE_URL + reverse("associate_account")
        adresse_user = mark_safe(adresse_base + "?part=user&token=" + self.token_user)
        adresse_user_ldap = mark_safe(adresse_base + "?part=ldap&token=" + self.token_user_ldap)
        c1 = {
            "login_inp_net": self.get_user_ldap().login_inp_net,
            "adresse": adresse_user,
        }
        c2 = {
            "adresse": adresse_user_ldap,
            "user": self.user,
            "user_ldap": self.get_user_ldap(),
        }

        contenu_user = render_to_string("emails/association_mail_user.txt", c1)
        contenu_user_ldap = render_to_string("emails/association_mail_user_ldap.txt", c2)

        mail_from = settings.DEFAULT_FROM_EMAIL
        send_mail(
            "Association de votre compte à un login INP-net",
            contenu_user,
            mail_from,
            [self.user.email],
            fail_silently=False,
        )
        send_mail(
            "Demande d'association de votre login INP-net sur billetterie.inpt.fr",
            contenu_user_ldap,
            mail_from,
            [self.get_user_ldap().mail],
            fail_silently=False,
        )
